/*****************************************************************************/
/*  RLReflect(x)                                                             */
/*  Reflect a line composed of WORDs ACATTed together, or strrev a WORD. */
/*  This should only be used on lines or words,                              */
/*  it will mess up a whole paragraph                                        */
/*  Based on SetUnderline in z08.c					     */
/*									     */
/*****************************************************************************/
#if WITHRL
#include "externs.h"
#include "hebrew.h"
/* This is all from z06.c						     */
#define	MAX_STACK	200			/* size of parser stacks     */
static	OBJECT		obj_stack[MAX_STACK];	/* stack of objects          */
static	int		otop = -1;		/* top of obj_stack          */
#define PushObj(x)							\
{ zz_hold = x;								\
  if( ++otop < MAX_STACK ) obj_stack[otop] = zz_hold;			\
  else Error(6, 1, "line too long to reverse",			\
    FATAL, &fpos(obj_stack[otop-1]));					\
}

#define PopObj()	obj_stack[otop--]
#define	ObjTop		obj_stack[otop]

void RLReflect(OBJECT x)
{ 
	OBJECT y, link;
	debug1(DGP, D, "RLReflect called on(%s)", EchoObject(x));
	if( type(x) == ACAT )
	{
		for( link = Down(x);  link != x;  link = NextDown(link) )
		{ 
			Child(y, link);
			PushObj(y);
		}
		while( Down(x) != x ) DeleteLink( Down(x) );
		while (otop > -1)
		{
			Link(x, PopObj());
		}
	}
	else if( type(x) == WORD || type(x) == QWORD )
	{
		StrRev(string(x));
		word_language(x) = 0;
	}
	else if (type(x) == HCAT)
	{
		for( link = Down(x);  link != x;  link = NextDown(link) )
		{ 
			Child(y, link);
			PushObj(y);
		}
		while( Down(x) != x ) DeleteLink( Down(x) );
		while (otop > -1)
		{
			Link(x, PopObj());
		}
		language(save_style(x)) = 0;
	}
	else 
	{
		debug0(DGP, D, "RLReflect called on other type of OBJECT");
		for( link = Down(x);  link != x;  link = NextDown(link) )
		{ 
			Child(y, link);
			if (IsInRLLanguage(y))
			RLReflect(y);
		}
	}
	/*language(save_style(x)) = 0; */ /*  So we don't get reflected again */
} /* end RLReflect */
void RLReflectLines(OBJECT x)
{
	OBJECT y, link;
	{ for( link = Down(x);  link != x;  link = NextDown(link) )
		{ Child(y, link);
			RLReflect(y);
		}
	}
}
FULL_CHAR *StrRev(FULL_CHAR *in)
{
	int i, j;
	FULL_CHAR temp;
	for(i = 0, j = strlen(in) -1; i < j; i++, j--)
	{
		temp = in[i];
		in[i] = in[j];
		in[j] = temp;
	}
	return in;
}
void RLReflectOneLine(OBJECT x)
{
	unsigned char	 otype = type(x); 
	OBJECT y, link;
	if (otype == ACAT)
	{
		RLReflect(x);
	}
	if (otype == WORD || otype == QWORD)
	{
		StrRev(string(x));
	}
	/* Go to all leaves, and reflect them if they are ACAT or [Q]WORD */
	else /*if (FALSE)  This has to be a switch, otherwise it messes up internal stuff */
	{
		for( link = Down(x);  link != x;  link = NextDown(link) )
		{ 
			Child(y, link);
			RLReflectOneLine(y);
		}
	}
}
/* Right-left languages must have canonical name containing "RightLeft" */
BOOLEAN IsInRLLanguage(OBJECT x)
{
	if(type(x) == WORD || type(x) == QWORD)
	{
		if(word_language(x) && strstr((char*)LanguageString(word_language(x)), "RightLeft"))
		{
			return TRUE;
		}
		return FALSE;
	}
	else if(
		language(save_style(x)) && 
		strstr((char*)LanguageString(language(save_style(x))), "RightLeft"))
	{
		return TRUE;
	}
	return FALSE;
}
#endif /* WITHRL */
